%  o o o o o o     
%    o o o o o                                           
%    o o o o o    VariSphear VSA                            
%    o o o o o    Variable Spherical Microphone Array  
%    o o o                                            
%                                                    
%    /// ASAR RESEARCH GROUP          
%
%    Cologne University of Applied Sciences         
%    Deutsche Telekom Laboratories, Berlin          
%    Technical University of Berlin                 
%    Westdeutscher Rundfunk, Cologne              
%    
%
%
% VariSphear DRV - Device Drivers : Version R11-0520 
% -----------------------------------------------------------------------
%
% Communication gateway to the VSA temperature sensor unit. 
%
% Device Type: Hygrosens TSIC-LABKIT-RS232 evaluation unit
%                        TSIC306-TO92 sensor 0.3K
% 
% /// Feel free to modify and share BUT: Always mark your changes please!
%
%
% [temp, flag] = vs_temp(COM)
%
% COM : COM Port, e.g. 'COM3' Type: String 
% 
% 
% temp      Returns the temperature in [C]
% flag      returns 0 if everything is ok.        
%
%           Error codes:
%                    
%           -1 : commnunication or data consistency error 
%           -2 : unit responds but the sensor is not connected
%
% ----------------------------------------------------------------------
%     
% Advice :       - Look up the COM-Port for your specific system ->  
%                  NPort Driver Software. The Laser is connected to 
%                  PORT #3 of the NPort 5410 Device Server.
%
% Dependencies :  VSAdeviceDriver.mexw32 (WIN32)
%                 VSAdeviceDriver.mexw64 (WIN64)
%
% Author        :  bBrn - Benny Bernschtz   
% Version       :  20M5.11 
% Last Version  :  V1.0 VariSphear FFL
% Since V1.0    :  - New lowlevel device driver
%                  - WARNING: SYNTAX IS NOT COMPATIBLE TO FFL v1.0



function [temp, flag] = vs_temp(COMPORT)

temp=0; 
flag=-1; %Default return error

if nargin==0
   disp('Error: No COM Port defined.');
   return
end

COMPORT=upper(COMPORT);
comNumber = str2double(strrep(COMPORT,'COM',''));

timeout=0;
while(1)
    
    [sensorErrorA, sensorResponseA] = VSAdeviceDriver(comNumber, 29, 1, 's', 2);
    if ~sensorErrorA
       [sensorErrorB, sensorResponseB] = VSAdeviceDriver(comNumber, 29, 1, 's', 2);
    end

    if ~sensorErrorA && ~sensorErrorB
        if sensorResponseA(3:5)==sensorResponseB(3:5) %Compare the two results
           flag=0;
           break
        end
    else
       temp=0;
       flag=-1;       
       disp('Sensor error or inconsistent data. Retry.');
       pause(0.1);
    end
    
    timeout=timeout+1;    
    if timeout > 12
       temp=0;
       flag=-1;
       return
    end   
    
end %end Error-Loop

sensorResponseA=sensorResponseA(3:6)-48;   % Extract Info from Sensor Input 1 (3:6), -48: ASCII decode

for i=1:length(sensorResponseA) % HEX Values A-F to Dec
    if sensorResponseA(i)>9 sensorResponseA(i)=sensorResponseA(i)-7;
    end 
end

sensorResponseA = 4096*sensorResponseA(1)+256*sensorResponseA(2)+16*sensorResponseA(3)+sensorResponseA(4); % HEX BYTES TO DECIMAL
sensorResponseA = (sensorResponseA/(10.235)-50)*10;                       % Calibration/Conversion to degree Celsius
sensorResponseA = round(sensorResponseA)/10;  % Cut off

temp=sensorResponseA;

%Catch Sensor connection bad error:
if sensorResponseA>6000
   disp('Error: Temperature eval unit responds - but the sensor is missing.');
   temp=0;
   flag=-2;
end

%%% CALIBRATION INFO FOR: Sensor TSIC306: 0=-50C, 2047=150C 
% Temp [C]=(VALUE/10.235)-50


